==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/client/integration/jei/PetalApothecaryRecipeCategory.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.client.integration.jei;

import com.mojang.blaze3d.systems.RenderSystem;
import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.gui.drawable.IDrawableStatic;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.recipe.category.IRecipeCategory;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_241;
import net.minecraft.class_2561;
import net.minecraft.class_4587;
import org.jetbrains.annotations.NotNull;

import vazkii.botania.api.recipe.PetalApothecaryRecipe;
import vazkii.botania.common.block.BotaniaBlocks;
import vazkii.botania.common.lib.LibMisc;

import static vazkii.botania.common.lib.ResourceLocationHelper.prefix;

public class PetalApothecaryRecipeCategory implements IRecipeCategory<PetalApothecaryRecipe> {

	public static final RecipeType<PetalApothecaryRecipe> TYPE = RecipeType.create(LibMisc.MOD_ID, "petals", PetalApothecaryRecipe.class);
	private final IDrawableStatic background;
	private final class_2561 localizedName;
	private final IDrawableStatic overlay;
	private final IDrawable icon;

	public PetalApothecaryRecipeCategory(IGuiHelper guiHelper) {
		background = guiHelper.createBlankDrawable(114, 97);
		localizedName = class_2561.method_43471("botania.nei.petalApothecary");
		overlay = guiHelper.createDrawable(prefix("textures/gui/petal_overlay.png"),
				17, 11, 114, 82);
		icon = guiHelper.createDrawableIngredient(VanillaTypes.ITEM_STACK, new class_1799(BotaniaBlocks.defaultAltar));
	}

	@NotNull
	@Override
	public RecipeType<PetalApothecaryRecipe> getRecipeType() {
		return TYPE;
	}

	@NotNull
	@Override
	public class_2561 getTitle() {
		return localizedName;
	}

	@NotNull
	@Override
	public IDrawable getBackground() {
		return background;
	}

	@NotNull
	@Override
	public IDrawable getIcon() {
		return icon;
	}

	@Override
	public void draw(@NotNull PetalApothecaryRecipe recipe, @NotNull IRecipeSlotsView slotsView, @NotNull class_4587 ms, double mouseX, double mouseY) {
		RenderSystem.enableBlend();
		overlay.draw(ms, 0, 4);
		RenderSystem.disableBlend();
	}

	@Override
	public void setRecipe(@NotNull IRecipeLayoutBuilder builder, @NotNull PetalApothecaryRecipe recipe, @NotNull IFocusGroup focusGroup) {
		builder.addSlot(RecipeIngredientRole.CATALYST, 48, 45)
				.addItemStack(new class_1799(BotaniaBlocks.defaultAltar));

		double angleBetweenEach = 360.0 / recipe.method_8117().size();
		class_241 point = new class_241(48, 13), center = new class_241(48, 45);

		for (var ingr : recipe.method_8117()) {
			builder.addSlot(RecipeIngredientRole.INPUT, (int) point.field_1343, (int) point.field_1342)
					.addIngredients(ingr);
			point = rotatePointAbout(point, center, angleBetweenEach);
		}

		builder.addSlot(RecipeIngredientRole.OUTPUT, 86, 10)
				.addItemStack(recipe.method_8110());
	}

	public static class_241 rotatePointAbout(class_241 in, class_241 about, double degrees) {
		double rad = degrees * Math.PI / 180.0;
		double newX = Math.cos(rad) * (in.field_1343 - about.field_1343) - Math.sin(rad) * (in.field_1342 - about.field_1342) + about.field_1343;
		double newY = Math.sin(rad) * (in.field_1343 - about.field_1343) + Math.cos(rad) * (in.field_1342 - about.field_1342) + about.field_1342;
		return new class_241((float) newX, (float) newY);
	}
}

==> Botania-1.20.1-451-FABRIC-20251124.130120-29-sources.jar: vazkii/botania/client/integration/jei/PetalApothecaryRecipeCategory.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.client.integration.jei;

import com.mojang.blaze3d.systems.RenderSystem;

import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.gui.drawable.IDrawableStatic;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.recipe.category.IRecipeCategory;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_2248;
import net.minecraft.class_241;
import net.minecraft.class_2561;
import net.minecraft.class_332;
import net.minecraft.class_5455;
import org.jetbrains.annotations.NotNull;

import vazkii.botania.api.recipe.PetalApothecaryRecipe;
import vazkii.botania.common.block.BotaniaBlocks;
import vazkii.botania.common.lib.LibMisc;

import java.util.List;

import static vazkii.botania.common.lib.ResourceLocationHelper.prefix;

public class PetalApothecaryRecipeCategory implements IRecipeCategory<PetalApothecaryRecipe> {

	public static final RecipeType<PetalApothecaryRecipe> TYPE = RecipeType.create(LibMisc.MOD_ID, "petals", PetalApothecaryRecipe.class);
	public static final int CENTER_X = 48;
	public static final int CENTER_Y = 45;
	private final IDrawableStatic background;
	private final class_2561 localizedName;
	private final IDrawableStatic overlay;
	private final IDrawable icon;
	private final class_1856 WATER_BUCKET = class_1856.method_8091(class_1802.field_8705);

	public PetalApothecaryRecipeCategory(IGuiHelper guiHelper) {
		background = guiHelper.createBlankDrawable(114, 97);
		localizedName = class_2561.method_43471("botania.nei.petalApothecary");
		overlay = guiHelper.createDrawable(prefix("textures/gui/petal_overlay.png"),
				17, 11, 114, 82);
		icon = guiHelper.createDrawableIngredient(VanillaTypes.ITEM_STACK, new class_1799(BotaniaBlocks.defaultAltar));
	}

	@NotNull
	@Override
	public RecipeType<PetalApothecaryRecipe> getRecipeType() {
		return TYPE;
	}

	@NotNull
	@Override
	public class_2561 getTitle() {
		return localizedName;
	}

	@NotNull
	@Override
	public IDrawable getBackground() {
		return background;
	}

	@NotNull
	@Override
	public IDrawable getIcon() {
		return icon;
	}

	@Override
	public void draw(@NotNull PetalApothecaryRecipe recipe, @NotNull IRecipeSlotsView slotsView, @NotNull class_332 gui, double mouseX, double mouseY) {
		RenderSystem.enableBlend();
		overlay.draw(gui, 0, 4);
		RenderSystem.disableBlend();
	}

	@Override
	public void setRecipe(@NotNull IRecipeLayoutBuilder builder, @NotNull PetalApothecaryRecipe recipe, @NotNull IFocusGroup focusGroup) {
		setRecipeLayout(builder, recipe.method_8117(), BotaniaBlocks.defaultAltar,
				recipe.method_8110(class_5455.field_40585), WATER_BUCKET, recipe.getReagent());
	}

	public static void setRecipeLayout(@NotNull IRecipeLayoutBuilder builder, List<class_1856> ingredients, class_2248 catalyst, class_1799 output, class_1856... reagents) {
		class_241 center = new class_241(CENTER_X, CENTER_Y);
		if (reagents.length > 0) {
			class_241 reagentPoint = new class_241(CENTER_X, CENTER_Y + 10);
			builder.addSlot(RecipeIngredientRole.CATALYST, (int) reagentPoint.field_1343, (int) reagentPoint.field_1342).addItemStack(new class_1799(catalyst));

			double angleBetweenReagents = 360.0 / (reagents.length + 1);
			for (int i = 0; i < reagents.length; i++) {
				reagentPoint = rotatePointAbout(reagentPoint, center, angleBetweenReagents);
				builder.addSlot(RecipeIngredientRole.INPUT, (int) reagentPoint.field_1343, (int) reagentPoint.field_1342).addIngredients(reagents[i]);
			}
		} else {
			builder.addSlot(RecipeIngredientRole.CATALYST, CENTER_X, CENTER_Y).addItemStack(new class_1799(catalyst));
		}
		double angleBetweenEach = 360.0 / ingredients.size();
		class_241 point = new class_241(CENTER_X, 13);

		for (var ingr : ingredients) {
			builder.addSlot(RecipeIngredientRole.INPUT, (int) point.field_1343, (int) point.field_1342).addIngredients(ingr);
			point = rotatePointAbout(point, center, angleBetweenEach);
		}

		// TODO 1.19.4 figure out the proper way to get a registry access
		builder.addSlot(RecipeIngredientRole.OUTPUT, 86, 10)
				.addItemStack(output);
	}

	public static class_241 rotatePointAbout(class_241 in, class_241 about, double degrees) {
		double rad = degrees * Math.PI / 180.0;
		double newX = Math.cos(rad) * (in.field_1343 - about.field_1343) - Math.sin(rad) * (in.field_1342 - about.field_1342) + about.field_1343;
		double newY = Math.sin(rad) * (in.field_1343 - about.field_1343) + Math.cos(rad) * (in.field_1342 - about.field_1342) + about.field_1342;
		return new class_241((float) newX, (float) newY);
	}
}
